@@ -79,8 +79,6 @@ $url_path = '/cacti/';
 /* disable log rotation setting */
 $disable_log_rotation = false;
 
-/* allow upto 5000 items to be selected */
-ini_set('max_input_vars', '5000');
 $config = array();
 
 /* Include configuration, or use the defaults */
@@ -194,6 +192,10 @@ $colors = array();
 /* this should be auto-detected, set it manually if needed */
 $config['cacti_server_os'] = (strstr(PHP_OS, 'WIN')) ? 'win32' : 'unix';
 
+if (!empty($path_csrf_secret)) {
+	$config['path_csrf_secret'] = $path_csrf_secret;
+}
+
 /* built-in snmp support */
 if (isset($php_snmp_support) && !$php_snmp_support) {
 	$config['php_snmp_support'] = false;
@@ -218,7 +220,7 @@ if (empty($database_port)) {
 }
 
 /* set URL path */
-if (! isset($url_path)) {
+if (!isset($url_path)) {
 	$url_path = '';
 }
 $config['url_path'] = $url_path;
@@ -259,7 +261,7 @@ include_once($config['include_path'] . '/global_constants.php');
 
 $filename = get_current_page();
 
-$config['is_web'] = true;
+$config['is_web'] = !defined('CACTI_CLI_ONLY');
 if ((isset($no_http_headers) && $no_http_headers == true) || in_array($filename, $no_http_header_files, true)) {
 	$config['is_web'] = false;
 }
@@ -387,32 +389,59 @@ if ($config['is_web']) {
 
 	/* set the maximum post size */
 	ini_set('post_max_size', '8M');
-	ini_set('max_input_vars', '5000');
-	ini_set('session.cookie_httponly', '1');
-	if (!empty($_SERVER['HTTPS']) && $_SERVER['HTTPS'] != 'off') {
-		ini_set('session.cookie_secure', '1');
+
+	/* add additional cookie directives */
+	ini_set('session.cookie_httponly', true);
+	ini_set('session.cookie_path', $config['url_path']);
+	ini_set('session.use_strict_mode', true);
+
+	$options = array(
+		'cookie_httponly' => true,
+		'cookie_path'     => $config['url_path'],
+		'use_strict_mode' => true
+	);
+
+	if (isset($cacti_cookie_domain) && $cacti_cookie_domain != '') {
+		ini_set('session.cookie_domain', $cacti_cookie_domain);
+		$options['cookie_domain'] = $cacti_cookie_domain;
 	}
 
+	// SameSite php7.3+ behavior
+	if (version_compare(PHP_VERSION, '7.3', '>=')) {
+		ini_set('session.cookie_samesite', 'Strict');
+		$options['cookie_samesite'] = 'Strict';
+	}
+
+	if (!empty($_SERVER['HTTPS']) && $_SERVER['HTTPS'] != 'off') {
+		ini_set('session.cookie_secure', true);
+		$options['cookie_secure'] = true;
+	}
+
+	$config['cookie_options']     = $options;
+	$config['cacti_session_name'] = $cacti_session_name;
+
 	/* we don't want these pages cached */
-	header('Expires: Mon, 26 Jul 1997 05:00:00 GMT');
 	header('Last-Modified: ' . gmdate('D, d M Y H:i:s') . ' GMT');
-	header('Cache-Control: no-store, no-cache, must-revalidate');
-	header('Cache-Control: post-check=0, pre-check=0', false);
-	header('Pragma: no-cache');
 	header('X-Frame-Options: SAMEORIGIN');
 
+	// SameSite legacy behavior
+	if (version_compare(PHP_VERSION, '7.3', '<')) {
+		header('Set-Cookie: cross-site-cookie=bar; SameSite=Strict;' . (isset($_SERVER['HTTPS']) && $_SERVER['HTTPS'] != 'off' ? ' Secure':''));
+	}
+
 	/* increased web hardening */
-	header("Content-Security-Policy: default-src *; img-src 'self' data: blob:; style-src 'self' 'unsafe-inline'; script-src 'self' 'unsafe-inline'; frame-ancestors 'self';");
+	$script_policy = read_config_option('content_security_policy_script');
+	if ($script_policy != '0' && $script_policy != '') {
+		$script_policy = "'$script_policy'";
+	}
+	header("Content-Security-Policy: default-src 'self'; img-src 'self' data: blob:; style-src 'self' 'unsafe-inline'; script-src 'self' $script_policy 'unsafe-inline'; frame-ancestors 'self'; worker-src 'self'");
 
 	/* prevent IE from silently rejects cookies sent from third party sites. */
 	header('P3P: CP="CAO PSA OUR"');
+	header('Cache-Control: no-store, no-cache, must-revalidate');
+	header('Cache-Control: max-age=31536000');
 
-	/* initialize php session */
-	if (!function_exists('session_name')) {
-		die('PHP Session Management is missing, please install PHP Session module');
-	}
-	session_name($cacti_session_name);
-	if (!session_id()) session_start();
+	cacti_session_start();
 
 	/* we never run with magic quotes on */
 	if (version_compare(PHP_VERSION, '5.4', '<=')) {
@@ -438,8 +467,7 @@ if ($config['is_web']) {
 		$_SESSION['cacti_cwd'] = $config['base_path'];
 	} else {
 		if ($_SESSION['cacti_cwd'] != $config['base_path']) {
-			session_unset();
-			session_destroy();
+			cacti_session_destroy();
 		}
 	}
 }
@@ -485,27 +513,13 @@ include_once($config['library_path'] . '/html_form.php');
 include_once($config['library_path'] . '/html_filter.php');
 include_once($config['library_path'] . '/variables.php');
 include_once($config['library_path'] . '/mib_cache.php');
+include_once($config['library_path'] . '/poller.php');
 include_once($config['library_path'] . '/snmpagent.php');
 include_once($config['library_path'] . '/aggregate.php');
 include_once($config['library_path'] . '/api_automation.php');
+include_once($config['include_path'] . '/csrf.php');
 
-/* cross site request forgery library */
 if ($config['is_web']) {
-	function csrf_startup() {
-		global $config;
-		csrf_conf('rewrite-js', $config['url_path'] . 'include/vendor/csrf/csrf-magic.js');
-		csrf_conf('callback', 'csrf_error_callback');
-		csrf_conf('expires', 7200);
-	}
-
-	function csrf_error_callback() {
-		raise_message('csrf_timeout');
-		header('Location: ' . sanitize_uri($_SERVER['REQUEST_URI']));
-		exit;
-	}
-
-	include_once($config['include_path'] . '/vendor/csrf/csrf-magic.php');
-
 	if (isset_request_var('newtheme')) {
 		unset($_SESSION['selected_theme']);
 	}
@@ -513,6 +527,20 @@ if ($config['is_web']) {
 	if (isset_request_var('csrf_timeout')) {
 		raise_message('csrf_ptimeout');
 	}
+
+	/* check for save actions using GET */
+	if (isset_request_var('action')) {
+		$action = get_nfilter_request_var('action');
+
+		$bad_actions = array('save', 'update_data', 'changepassword');
+
+		foreach($bad_actions as $bad) {
+			if ($action == $bad && !isset($_POST['__csrf_magic'])) {
+				cacti_log('WARNING: Attempt to use GET method for POST operations from IP ' . get_client_addr(), false, 'WEBUI');
+				exit;
+			}
+		}
+	}
 }
 
 api_plugin_hook('config_insert');
